#include "msp430x22x4.h"
#include "SoftBaughTypes.h"
#include "ADC10.h"

static __no_init SADC10_Info const * mADC10_ptHostInfo;
static __no_init PFN_STATE_VOID_VOID * mADC10_ppfnHostState;

// States
void ADC10_State_WaitForComplete( void );

// Helpers
void ADC10_Enter_WaitForComplete( void );
void ADC10_PrepareChannel( BYTE byChannel );
void ADC10_DeclareConversionComplete( void );
void ADC10_PowerDown( void );

static const WORD mADC10_wADC10CTL1[] =
{	INCH_0|CONSEQ_0,	// A0
	INCH_1|CONSEQ_0,	// A1
	INCH_2|CONSEQ_0,	// A2
	INCH_3|CONSEQ_0,	// A3
	INCH_4|CONSEQ_0,	// A4
	INCH_5|CONSEQ_0,	// A5
	INCH_6|CONSEQ_0,	// A6
	INCH_7|CONSEQ_0,	// A7
	INCH_8|CONSEQ_0,	// A8
	INCH_9|CONSEQ_0,	// A9
	INCH_10|CONSEQ_0,	// A10
	INCH_11|CONSEQ_0,	// A11
	INCH_12|CONSEQ_0,	// A12
	INCH_13|CONSEQ_0,	// A13
	INCH_14|CONSEQ_0,	// A14
	INCH_15|CONSEQ_0,	// A15
};

static const BYTE mADC10_byADC10AE0[] =
{	BIT0,	// A0
	BIT1,	// A1
	BIT2,	// A2
	BIT3,	// A3
	BIT4,	// A4
	BIT5,	// A5
	BIT6,	// A6
	BIT7,	// A7
};

static const BYTE mADC10_byADC10AE1[] =
{	BIT0,	// A8
	BIT1,	// A9
	BIT2,	// A10
	BIT3,	// A11
	BIT4,	// A12
	BIT5,	// A13
	BIT6,	// A14
	BIT7,	// A15
};

void ADC10_InitializeHW( void )
{
}

void ADC10_InitializeApp( void )
{
}

void ADC10_Entry( SADC10_Info const * ptADC10_Info, BYTE byChannel )
{
	// Save the host information
	mADC10_ptHostInfo = ptADC10_Info;
	mADC10_ppfnHostState = mADC10_ptHostInfo->ppfnHostState;

	// Prepare channel
	ADC10_PrepareChannel( byChannel );

	ADC10_Enter_WaitForComplete();
}

void ADC10_Enter_WaitForComplete( void )
{
	// Ask host to suspend ticks
	mADC10_ptHostInfo->pfnCallback_OnSuspendTicks();

	// Transition
	*mADC10_ppfnHostState = ADC10_State_WaitForComplete;

	// Start a conversion and enable interupts
	ADC10CTL0 |= (ADC10IE|ADC10SC);
}

void ADC10_State_WaitForComplete( void )
{	WORD wMask, wValue;

	wMask = ADC10CTL1;
	if( wMask & ADC10BUSY )
	// Not done yet
	{
		// Call idle callback
		mADC10_ptHostInfo->pfnCallback_OnIdle();
	
		return;
	}

	// Capture the value
	wValue = ADC10MEM;

	// Notify the host
	mADC10_ptHostInfo->pfnCallback_OnExit( wValue );
}

void ADC10_PrepareChannel( BYTE byChannel )
// Caller must ensure byChannel does not exceed 0x0F
{
	// Configure ADC measurements

	// Disable conversions to enable changes
	ADC10CTL0 &= ~(ENC);

	// Prepare ADC10 for sampling versus VCC on the desired channel
	ADC10CTL0 = (SREF_0|ADC10SHT_1|ADC10ON);
	ADC10CTL1 = mADC10_wADC10CTL1[byChannel];

	// Enable analog inputs on the desired channel
	ADC10AE0 = mADC10_byADC10AE0[byChannel];
	ADC10AE1 = mADC10_byADC10AE1[byChannel];

	// Enable conversions
	ADC10CTL0 |= (ENC);
}

void ADC10_DeclareConversionComplete( void )
{
	// Wake the host
	mADC10_ptHostInfo->pfnCallback_OnResumeTicks();
}

void ADC10_PowerDown( void )
{
	// Disable conversions to enable changes
	ADC10CTL0 &= ~(ENC);

	// Power-down ADC10
	ADC10CTL0 = (0x0000);

	// Disable analog inputs
	ADC10AE0 = 0x00;
	ADC10AE1 = 0x00;

	// Enable conversions
	ADC10CTL0 |= (ENC);
}

#pragma vector=ADC10_VECTOR
__interrupt void ADC10_ISR( void )
{
	ADC10_DeclareConversionComplete();
}

