#include "msp430x22x4.h"
#include "SoftBaughTypes.h"
#include "DAC7512.h"

#define DACCS_BIT			BIT1
#define DACCS_PORT			P1OUT

void DAC7512_SendValueSPI( BYTE byValue, BOOL bWaitComplete );

void DAC7512_InitializeHW( void )
{
	// Setup UCB0 for SPI from SMCLK / 2

	// Set UCSWRST
	UCB0CTL1 |= UCSWRST;

	// Initialize registers
	UCB0CTL0 = (UCCKPH|UCCKPL|UCMSB|UCMST|UCSYNC);
	UCB0CTL1 = (UCSSEL1|UCSWRST);
	UCB0BR0 = 0x02;
	UCB0BR1 = 0x00;
	UCB0STAT = 0x00;

	// Clear flags
	IFG2 &= ~(UCB0RXIFG);
	IFG2 |= (UCB0TXIFG);

	// Clear UCSWRST
	UCB0CTL1 &= ~UCSWRST;

	// We will use the DAC in polled mode, so no interrupts
}

void DAC7512_InitializeApp( void )
{
}

void DAC7512_SetValue( WORD wValue )
{
	// Enable DAC
	DACCS_PORT &= ~(DACCS_BIT);

	// Send high value
	DAC7512_SendValueSPI( HIBYTE( wValue ), FALSE );

	// Send low value
	DAC7512_SendValueSPI( LOBYTE( wValue ), TRUE );

	// Disable DAC
	DACCS_PORT |= (DACCS_BIT);
}

void DAC7512_SendValueSPI( BYTE byValue, BOOL bWaitComplete )
{
	// Make sure Tx buffer is available
	while( !(IFG2 & UCB0TXIFG) )
	{
	}

	// Send the value
	UCB0TXBUF = byValue;

	// Check whether to hold for completion
	if( bWaitComplete )
	{	
		while( UCB0STAT & UCBUSY )
		{
		}
	}
}
