#include "msp430x22x4.h"
#include "SoftBaughTypes.h"
#include "Monitor.h"
#include "UCA0_UART.h"
#include "Keyboard.h"

#define MONITOR_MIN_TX_QUEUE	32

static __no_init SMonitorInfo const * mMonitor_ptHostInfo;
static __no_init PFN_STATE_VOID_VOID * mMonitor_ppfnHostState;
static __no_init WORD mMonitor_wIndex;

// States
void Monitor_State_WaitForCommand( void );
void Monitor_State_SendMenu( void );

// Helper Functions
void Monitor_Enter_WaitForCommand( void );
void Monitor_Enter_SendMenu( void );
void Monitor_ShowPrompt( void );

// Event Sinks
void Monitor_RxEventSink( void );

// Menu strings
static const char mMonitor_szMenu1[] = "Q - Quit to host\n";
static const char mMonitor_szMenu2[] = "? - Show this long menu\n";
// Add more menu strings here

// Menu array
static const PCSZ mMonitor_aszMenu[] =
{	mMonitor_szMenu1,
	mMonitor_szMenu2,
	// And add the string pointers here
};
static const WORD mMonitor_wMenuStringCount =
	sizeof(mMonitor_aszMenu)/sizeof(char *);

void Monitor_InitializeHW( void )
{
}

void Monitor_InitializeApp( void )
{
}

// Event Sinks
void Monitor_RxEventSink( void )
{
	// Request the host to resume ticks
	mMonitor_ptHostInfo->pfnCallback_OnResumeTicks();
}

void Monitor_Entry( SMonitorInfo const * ptMonitorInfo )
{
	// Register the receiver event sink
	UCA0_UART_RegisterRxEventSink( Monitor_RxEventSink );

	// Flush the receive queue
	UCA0_UART_FlushRxQueue();

	// Show the banner
	UCA0_UART_QueueTxString( "\n\nBaby Monitor v1.0\n" );

	// Save the host information
	mMonitor_ptHostInfo = ptMonitorInfo;
	mMonitor_ppfnHostState = mMonitor_ptHostInfo->ppfnHostState;

	// Transition to waiting for commands
	Monitor_Enter_WaitForCommand();
}

void Monitor_ShowPrompt( void )
{
	UCA0_UART_QueueTxString( "\n:" );
}

// States
void Monitor_Enter_WaitForCommand( void )
{
	// Transition
	*mMonitor_ppfnHostState = Monitor_State_WaitForCommand;

	// Show the prompt
	Monitor_ShowPrompt();
}

void Monitor_State_WaitForCommand( void )
{	WORD wRx;

	wRx = UCA0_UART_GetNextRxValueEcho();
	if( NO_RX_AVAILABLE == wRx )
	// No character received
	{
		// Request the host to suspend ticks
		mMonitor_ptHostInfo->pfnCallback_OnSuspendTicks();
		return;
	}
	
	if( ('q' == wRx) || ('Q' == wRx) )
	// Quit to the host application
	{
		UCA0_UART_RegisterRxEventSink( NULL );
		mMonitor_ptHostInfo->pfnCallback_OnExit();
		return;
	}
	
	if( '?' == wRx )
	// Show the menu
	{
		Monitor_Enter_SendMenu();
		return;
	}
	
	// Everything else is invalid
	UCA0_UART_QueueTxString( " Invalid command, ? for help\n" );
	Monitor_ShowPrompt();
}

void Monitor_Enter_SendMenu( void )
{
	// Initialize the string index
	mMonitor_wIndex = 0;
	
	// Send the command title
	UCA0_UART_QueueTxString( "\nSupported commands:\n\n" );
	
	// Transition states
	*mMonitor_ppfnHostState = Monitor_State_SendMenu;
}

void Monitor_State_SendMenu( void )
{	WORD wAvailableTxQueue;

	// Check whether room is available for a new string
	wAvailableTxQueue = UCA0_UART_GetAvailableTxQueue();
	if( wAvailableTxQueue < MONITOR_MIN_TX_QUEUE )
	// Not enough room yet
	{	return;
	}
	
	// Check whether done with menu
	if( mMonitor_wIndex >= mMonitor_wMenuStringCount )
	// Sent the last string already
	{
		// Transition back to waiting for the next command
		Monitor_Enter_WaitForCommand();
		return;
	}

	// Send the next string
	UCA0_UART_QueueTxString( mMonitor_aszMenu[mMonitor_wIndex++] );
}

