#include "msp430x22x4.h"
#include "SoftBaughTypes.h"
#include "System.h"
#include "MyApp.h"
#include "Switches.h"
#include "Monitor.h"

#define APP_TICK_DIVISOR		64
#define MAX_CHUNK_TICKS			((0x10000/TICK_DIVISOR)-2)

typedef void ( * PFN_MONITOR_OPTION_VOID_VOID )( void );
// Example:
// void MyMonitorOption( void );

static __no_init WORD mMyApp_wCounter;
static __no_init WORD mMyApp_wRemainingDelayTicks;
__no_init PFN_STATE_VOID_VOID gMyApp_pfnCurrentState;
static __no_init PFN_STATE_VOID_VOID mMyApp_pfnNextStateAfterDelay;
static __no_init PFN_STATE_VOID_VOID mMyApp_pfnOnSW1;
static __no_init PFN_MONITOR_OPTION_VOID_VOID mMyApp_pfnMonitorStartOption;

// App states
void MyApp_State_Delay( void );
void MyApp_State_NextDelayChunk( void );
void MyApp_State_TurnOnLED( void );
void MyApp_State_TurnOffLED( void );

// OnSW1 states
void MyApp_StateOnSW1_ToggleD3( void );
void MyApp_StateOnSW1_ToggleD4( void );

// Event sinks
void MyApp_OnSW1( void );
void MyApp_EventSink_ToggleD3( void );
void MyApp_EventSink_ToggleD4( void );

// Monitor options
void MyApp_StartMonitor( void );
void MyApp_SkipMonitor( void );

// Helper functions
void MyApp_Enter_Delay( PFN_STATE_VOID_VOID	pfnNextState,
					   	WORD wDelayTicks );
void MyApp_OnMonitorExit( void );
void MyApp_OnMonitorSuspendTicks( void );
void MyApp_OnMonitorResumeTicks( void );

static const SMonitorInfo mMyApp_tMonitorInfo =
{	&gMyApp_pfnCurrentState,
	MyApp_OnMonitorExit,
	MyApp_OnMonitorSuspendTicks,
	MyApp_OnMonitorResumeTicks
};

void MyApp_InitializeHW( void )
{
	// Prepare the first interrupt
	TACCR0 = TICK_DIVISOR;
}

void MyApp_InitializeApp( void )
{
	mMyApp_pfnMonitorStartOption = MyApp_SkipMonitor;

	mMyApp_pfnOnSW1 = MyApp_StateOnSW1_ToggleD4;

	Switches_RegisterEventSinkSW1( MyApp_OnSW1 );
	Switches_RegisterEventSinkSW2( MyApp_EventSink_ToggleD3 );

	mMyApp_wCounter = APP_TICK_DIVISOR;
	gMyApp_pfnCurrentState = MyApp_State_TurnOffLED;
}

void MyApp_Tick( void )
{
	gMyApp_pfnCurrentState();
}

void MyApp_OnSW1( void )
{
	// Set monitor to start
	mMyApp_pfnMonitorStartOption = MyApp_StartMonitor;

	// Call the switch option
	mMyApp_pfnOnSW1();
}

// OnSW1 states
void MyApp_StateOnSW1_ToggleD3( void )
{
	// Clear D2
	P1OUT &= ~(BIT3);

	// Change the SW2 registration
	Switches_RegisterEventSinkSW2( MyApp_EventSink_ToggleD3 );

	// Setup the next state
	mMyApp_pfnOnSW1 = MyApp_StateOnSW1_ToggleD4;
}

void MyApp_StateOnSW1_ToggleD4( void )
{
	// Set D2
	P1OUT |= (BIT3);

	// Change the SW2 registration
	Switches_RegisterEventSinkSW2( MyApp_EventSink_ToggleD4 );

	// Setup the next state
	mMyApp_pfnOnSW1 = MyApp_StateOnSW1_ToggleD3;
}

void MyApp_EventSink_ToggleD3( void )
{
	// Toggle P3.6
	P3OUT ^= BIT6;
}

void MyApp_EventSink_ToggleD4( void )
{
	// Toggle P3.7
	P3OUT ^= BIT7;
}

void MyApp_Enter_Delay( PFN_STATE_VOID_VOID	pfnNextState,
					   	WORD wDelayTicks )
{	WORD wOffsetCCR0;

	// Prepare next state
	gMyApp_pfnCurrentState = pfnNextState;

	if( 0 == wDelayTicks )
	{	return;
	}

	if( 1 == wDelayTicks )
	{	TACCR0 += TICK_DIVISOR;
		return;
	}

	if( wDelayTicks > MAX_CHUNK_TICKS )
	// Need to break into pieces
	{	// Calculate offset
		wOffsetCCR0 = TICK_DIVISOR * MAX_CHUNK_TICKS;
		// Calculate ticks remaining, accounting for the chunk update tick
		mMyApp_wRemainingDelayTicks = wDelayTicks - (MAX_CHUNK_TICKS+1);
		// This value could be zero now
		
		// Prepare chunk update state	
		mMyApp_pfnNextStateAfterDelay = pfnNextState;
		gMyApp_pfnCurrentState = MyApp_State_NextDelayChunk;
	}
	else
	// This can be handled in one gulp
	{	// Calculate offset
		wOffsetCCR0 = TICK_DIVISOR * wDelayTicks;
		// Next state does not need updating
	}

	// Add offset as a virtual delay
	TACCR0 += wOffsetCCR0;

//	// Prepare next state
//	mMyApp_wCounter = wDelayTicks;
//	mMyApp_pfnNextStateAfterDelay = pfnNextState;
//	gMyApp_pfnCurrentState = MyApp_State_Delay;
}

void MyApp_State_NextDelayChunk( void )
{
	MyApp_Enter_Delay(	mMyApp_pfnNextStateAfterDelay,
					   	mMyApp_wRemainingDelayTicks );
}

void MyApp_State_Delay( void )
{
	// Update count
	mMyApp_wCounter--;
	if( mMyApp_wCounter > 0 )
	// Count not expired yet
	{	return;
	}

	// Transition
	gMyApp_pfnCurrentState = mMyApp_pfnNextStateAfterDelay;
}

void MyApp_State_TurnOnLED( void )
{
	// Set LED
	P1OUT |= (BIT0);

	// Transition
	MyApp_Enter_Delay( MyApp_State_TurnOffLED, APP_TICK_DIVISOR-1 );
}

void MyApp_State_TurnOffLED( void )
{
	// Clear LED
	P1OUT &= ~(BIT0);

	// Optionally start the monitor application
	mMyApp_pfnMonitorStartOption();	
}

void MyApp_StartMonitor( void )
{
	// Launch monitor application
	Monitor_Entry( &mMyApp_tMonitorInfo );
}

void MyApp_SkipMonitor( void )
{
	// Transition
	MyApp_Enter_Delay( MyApp_State_TurnOnLED, APP_TICK_DIVISOR-1 );
}

void MyApp_OnMonitorExit( void )
// Called by the monitor application when it exits
{
	// Transition
	MyApp_Enter_Delay( MyApp_State_TurnOnLED, APP_TICK_DIVISOR-1 );

	// Configure the skip option
	mMyApp_pfnMonitorStartOption = MyApp_SkipMonitor;
}

void MyApp_OnMonitorSuspendTicks( void )
// Called by the monitor application to suspend further ticks
{
	// Simply turn off the CCR0 interrupt
	TACCTL0 &= ~(CCIE);
}

void MyApp_OnMonitorResumeTicks( void )
// Called by the monitor application to resume ticks
{
	if( 0 == (TACCTL0 & CCIE) )
	// Ticks are suspended
	{	// Update TACCR0 to allow the TAR to count to the TACCR0 value
		TACCR0 = TAR + 2;
		// Clear the interrupt flag
		TACCTL0 &= ~(CCIFG);
		// Enable the interrupt
		TACCTL0 |= (CCIE);
	}
}

#pragma vector=TIMERA0_VECTOR
__interrupt void TimerA_CCR0_ISR( void )
{
	// Update for the next interrupt
	TACCR0 += TICK_DIVISOR;

	// Wake the processor
	__low_power_mode_off_on_exit();
}

