#include "msp430x22x4.h"
#include "SoftBaughTypes.h"
#include "System.h"

#define APP_TICK_DIVISOR		64

static __no_init int mMain_nCounter;
static __no_init BOOL mMain_bStateLED;

void InitializeHW( void );
void InitializeApps( void );
void TickApplication( void );

void main( void )
{
	// Stop watchdog timer to prevent time out reset
	WDTCTL = (WDTPW | WDTHOLD);

	InitializeHW();
	InitializeApps();
	__enable_interrupt();

	while( 1 )
	{
		__low_power_mode_3();
		TickApplication();
	}
}

void InitializeHW( void )
{
	System_InitializeHW();
}

void InitializeApps( void )
{
	mMain_nCounter = APP_TICK_DIVISOR;
	mMain_bStateLED = FALSE;
}

void TickApplication( void )
{
	mMain_nCounter--;
	if( mMain_nCounter > 0 )
	// Count not expired yet
	{	return;
	}
	// Restart the counter
	mMain_nCounter = APP_TICK_DIVISOR;

	// Update the LED
	if( mMain_bStateLED )
	// LED is set, clear it
	{	P1OUT &= ~(BIT0);
		mMain_bStateLED = FALSE;
	}
	else
	// LED is clear, set it
	{	P1OUT |= (BIT0);
		mMain_bStateLED = TRUE;
	}
}

#pragma vector=TIMERA1_VECTOR
__interrupt void TimerA3_ISR( void )
// Shared interrupt vector for TAR, CCR1 and CCR2
{
	switch ( __even_in_range(TAIV, 10) )
	{
		case  2:
		// TACCR1
			break;
		
		case  4:
		// TACCR2
			__low_power_mode_off_on_exit();
			break;
			
		case 10:
		// TAIFG
			// Wake the processor
			__low_power_mode_off_on_exit();
			break;
	}
}
