#include "msp430x22x4.h"
#include "SoftBaughTypes.h"
#include "UserEntry.h"
#include "UCA0_UART.h"
#include "Keyboard.h"
#include "UserEntryTable.h"

#define DEFAULT_WORD		0xFFFF
#define DEFAULT_BYTE		0xFF

typedef void ( * PFN_USER_ENTRY_HANDLER_VOID_BYTE )( BYTE );
// Example:
// void UserEntry_Handler_SomeInput( BYTE byIn );
typedef PFN_USER_ENTRY_HANDLER_VOID_BYTE const * PCPFN_USER_ENTRY_HANDLER_VOID_BYTE;

static __no_init SUserEntryInfo const * mUserEntry_ptHostInfo;
static __no_init PCPFN_USER_ENTRY_HANDLER_VOID_BYTE mUserEntry_pcapfnCurrentState;
static __no_init BYTE mUserEntry_byNybble3;
static __no_init BYTE mUserEntry_byNybble2;
static __no_init BYTE mUserEntry_byNybble1;
static __no_init BYTE mUserEntry_byNybble0;

// Format Functions
WORD UserEntry_FormatWORD_3210( void );
WORD UserEntry_FormatWORD_321( void );
WORD UserEntry_FormatWORD_32( void );
WORD UserEntry_FormatWORD_3( void );
WORD UserEntry_FormatBYTE_10( void );
WORD UserEntry_FormatBYTE_1( void );

// Handlers
void UserEntry_Handler_OnExcessInput( BYTE byIn );
void UserEntry_Handler_OnInvalid( BYTE byIn );
void UserEntry_Handler_OnDigit_3W( BYTE byIn );
void UserEntry_Handler_OnDigit_2W( BYTE byIn );
void UserEntry_Handler_OnDigit_1W( BYTE byIn );
void UserEntry_Handler_OnDigit_0W( BYTE byIn );
void UserEntry_Handler_OnDigit_1B( BYTE byIn );
void UserEntry_Handler_OnDigit_0B( BYTE byIn );
void UserEntry_Handler_OnUpperAF_3W( BYTE byIn );
void UserEntry_Handler_OnUpperAF_2W( BYTE byIn );
void UserEntry_Handler_OnUpperAF_1W( BYTE byIn );
void UserEntry_Handler_OnUpperAF_0W( BYTE byIn );
void UserEntry_Handler_OnUpperAF_1B( BYTE byIn );
void UserEntry_Handler_OnUpperAF_0B( BYTE byIn );
void UserEntry_Handler_OnLowerAF_3W( BYTE byIn );
void UserEntry_Handler_OnLowerAF_2W( BYTE byIn );
void UserEntry_Handler_OnLowerAF_1W( BYTE byIn );
void UserEntry_Handler_OnLowerAF_0W( BYTE byIn );
void UserEntry_Handler_OnLowerAF_1B( BYTE byIn );
void UserEntry_Handler_OnLowerAF_0B( BYTE byIn );
void UserEntry_Handler_OnMore_2W( BYTE byIn );
void UserEntry_Handler_OnMore_1W( BYTE byIn );
void UserEntry_Handler_OnMore_0W( BYTE byIn );
void UserEntry_Handler_OnMore_0B( BYTE byIn );
void UserEntry_Handler_OnBackspace_Ignore( BYTE byIn );
void UserEntry_Handler_OnBackspace_2W( BYTE byIn );
void UserEntry_Handler_OnBackspace_1W( BYTE byIn );
void UserEntry_Handler_OnBackspace_0W( BYTE byIn );
void UserEntry_Handler_OnBackspace_0B( BYTE byIn );
void UserEntry_Handler_OnDone_3W( BYTE byIn );
void UserEntry_Handler_OnDone_2W( BYTE byIn );
void UserEntry_Handler_OnDone_1W( BYTE byIn );
void UserEntry_Handler_OnDone_0W( BYTE byIn );
void UserEntry_Handler_OnDone_1B( BYTE byIn );
void UserEntry_Handler_OnDone_0B( BYTE byIn );
void UserEntry_Handler_OnEscape( BYTE byIn );

// Input Handler Tables
static const PFN_USER_ENTRY_HANDLER_VOID_BYTE
	mUserEntry_apfnHandlers_Idle[CHAR_TYPE_COUNT] =
{	UserEntry_Handler_OnExcessInput,
	UserEntry_Handler_OnExcessInput,
	UserEntry_Handler_OnExcessInput,
	UserEntry_Handler_OnExcessInput,
	UserEntry_Handler_OnExcessInput,
	UserEntry_Handler_OnExcessInput,
	UserEntry_Handler_OnExcessInput,
	UserEntry_Handler_OnExcessInput,
	UserEntry_Handler_OnExcessInput,
};

static const PFN_USER_ENTRY_HANDLER_VOID_BYTE
	mUserEntry_apfnHandlers_3W[CHAR_TYPE_COUNT] =
{	UserEntry_Handler_OnInvalid,
	UserEntry_Handler_OnDigit_3W,
	UserEntry_Handler_OnUpperAF_3W,
	UserEntry_Handler_OnLowerAF_3W,
	UserEntry_Handler_OnInvalid,
	UserEntry_Handler_OnBackspace_Ignore,
	UserEntry_Handler_OnDone_3W,
	UserEntry_Handler_OnEscape,
	UserEntry_Handler_OnInvalid,
};

static const PFN_USER_ENTRY_HANDLER_VOID_BYTE
	mUserEntry_apfnHandlers_2W[CHAR_TYPE_COUNT] =
{	UserEntry_Handler_OnInvalid,
	UserEntry_Handler_OnDigit_2W,
	UserEntry_Handler_OnUpperAF_2W,
	UserEntry_Handler_OnLowerAF_2W,
	UserEntry_Handler_OnMore_2W,
	UserEntry_Handler_OnBackspace_2W,
	UserEntry_Handler_OnDone_2W,
	UserEntry_Handler_OnEscape,
	UserEntry_Handler_OnMore_2W,
};

static const PFN_USER_ENTRY_HANDLER_VOID_BYTE
	mUserEntry_apfnHandlers_1W[CHAR_TYPE_COUNT] =
{	UserEntry_Handler_OnInvalid,
	UserEntry_Handler_OnDigit_1W,
	UserEntry_Handler_OnUpperAF_1W,
	UserEntry_Handler_OnLowerAF_1W,
	UserEntry_Handler_OnMore_1W,
	UserEntry_Handler_OnBackspace_1W,
	UserEntry_Handler_OnDone_1W,
	UserEntry_Handler_OnEscape,
	UserEntry_Handler_OnMore_1W,
};

static const PFN_USER_ENTRY_HANDLER_VOID_BYTE
	mUserEntry_apfnHandlers_0W[CHAR_TYPE_COUNT] =
{	UserEntry_Handler_OnInvalid,
	UserEntry_Handler_OnDigit_0W,
	UserEntry_Handler_OnUpperAF_0W,
	UserEntry_Handler_OnLowerAF_0W,
	UserEntry_Handler_OnMore_0W,
	UserEntry_Handler_OnBackspace_0W,
	UserEntry_Handler_OnDone_0W,
	UserEntry_Handler_OnEscape,
	UserEntry_Handler_OnMore_0W,
};

static const PFN_USER_ENTRY_HANDLER_VOID_BYTE
	mUserEntry_apfnHandlers_1B[CHAR_TYPE_COUNT] =
{	UserEntry_Handler_OnInvalid,
	UserEntry_Handler_OnDigit_1B,
	UserEntry_Handler_OnUpperAF_1B,
	UserEntry_Handler_OnLowerAF_1B,
	UserEntry_Handler_OnInvalid,
	UserEntry_Handler_OnBackspace_Ignore,
	UserEntry_Handler_OnDone_1B,
	UserEntry_Handler_OnEscape,
	UserEntry_Handler_OnInvalid,
};

static const PFN_USER_ENTRY_HANDLER_VOID_BYTE
	mUserEntry_apfnHandlers_0B[CHAR_TYPE_COUNT] =
{	UserEntry_Handler_OnInvalid,
	UserEntry_Handler_OnDigit_0B,
	UserEntry_Handler_OnUpperAF_0B,
	UserEntry_Handler_OnLowerAF_0B,
	UserEntry_Handler_OnMore_0B,
	UserEntry_Handler_OnBackspace_0B,
	UserEntry_Handler_OnDone_0B,
	UserEntry_Handler_OnEscape,
	UserEntry_Handler_OnMore_0B,
};

void UserEntry_InitializeHW( void )
{
}

void UserEntry_InitializeApp( void )
{
	mUserEntry_pcapfnCurrentState = mUserEntry_apfnHandlers_Idle;
}

void UserEntry_EntryWORD( SUserEntryInfo const * ptUserEntryInfo )
{
	// Save the host information structure
	mUserEntry_ptHostInfo = ptUserEntryInfo;
	
	// And set the initial state
	mUserEntry_pcapfnCurrentState = mUserEntry_apfnHandlers_3W;
}

void UserEntry_EntryBYTE( SUserEntryInfo const * ptUserEntryInfo )
{
	// Save the host information structure
	mUserEntry_ptHostInfo = ptUserEntryInfo;
	
	// And set the initial state
	mUserEntry_pcapfnCurrentState = mUserEntry_apfnHandlers_1B;
}

void UserEntry_ConsumeBYTE( BYTE byIn )
{	BYTE byValueType;
	
	// Call the current state
	byValueType = UserEntryTable_ConvertBYTE( byIn );
	(mUserEntry_pcapfnCurrentState[byValueType])( byIn );
}

// Format Functions
WORD UserEntry_FormatWORD_3210( void )
{	WORD wValue;
	BYTE byHi, byLo;
	
	byHi = ((0x0F&mUserEntry_byNybble3)<<4) | (0x0F&mUserEntry_byNybble2);
	byLo = ((0x0F&mUserEntry_byNybble1)<<4) | (0x0F&mUserEntry_byNybble0);
	wValue = MAKEWORD( byLo, byHi );
	return wValue;
}

WORD UserEntry_FormatWORD_321( void )
{	WORD wValue;
	BYTE byHi, byLo;
	
	byHi = (0x0F&mUserEntry_byNybble3);
	byLo = ((0x0F&mUserEntry_byNybble2)<<4) | (0x0F&mUserEntry_byNybble1);
	wValue = MAKEWORD( byLo, byHi );
	return wValue;
}

WORD UserEntry_FormatWORD_32( void )
{	WORD wValue;
	
	wValue = ((0x0F&mUserEntry_byNybble3)<<4) | (0x0F&mUserEntry_byNybble2);
	return wValue;
}

WORD UserEntry_FormatWORD_3( void )
{	WORD wValue;
	
	wValue = (0x0F&mUserEntry_byNybble3);
	return wValue;
}

WORD UserEntry_FormatBYTE_10( void )
{	WORD wValue;

	wValue = ((0x0F&mUserEntry_byNybble1)<<4) | (0x0F&mUserEntry_byNybble0);
	return wValue;
}

WORD UserEntry_FormatBYTE_1( void )
{	WORD wValue;

	wValue = (0x0F&mUserEntry_byNybble1);
	return wValue;
}

// Handlers
void UserEntry_Handler_OnExcessInput( BYTE byIn )
{
	// Report this value to host
	mUserEntry_ptHostInfo->pfnCallback_OnExcessInput( byIn );
}

void UserEntry_Handler_OnInvalid( BYTE byIn )
{	
	// Erase the terminal character
	UCA0_UART_QueueTxErasingBackspace();
	
	// Queue a warning beep
	UCA0_UART_QueueTxChar( CHAR_BELL );
}

void UserEntry_Handler_OnDigit_3W( BYTE byIn )
// User has typed within range 0 through 9
{
	// Store this value
	mUserEntry_byNybble3 = byIn - '0';

	// Transition to next state
	mUserEntry_pcapfnCurrentState = mUserEntry_apfnHandlers_2W;
}

void UserEntry_Handler_OnDigit_2W( BYTE byIn )
// User has typed within range 0 through 9
{
	// Store this value
	mUserEntry_byNybble2 = byIn - '0';

	// Transition to next state
	mUserEntry_pcapfnCurrentState = mUserEntry_apfnHandlers_1W;
}

void UserEntry_Handler_OnDigit_1W( BYTE byIn )
// User has typed within range 0 through 9
{
	// Store this value
	mUserEntry_byNybble1 = byIn - '0';

	// Transition to next state
	mUserEntry_pcapfnCurrentState = mUserEntry_apfnHandlers_0W;
}

void UserEntry_Handler_OnDigit_0W( BYTE byIn )
// User has typed within range 0 through 9
{	WORD wValue;

	// Store this value
	mUserEntry_byNybble0 = byIn - '0';

	// Format the completed value
	wValue = UserEntry_FormatWORD_3210();
	
	// Transition to next state
	mUserEntry_pcapfnCurrentState = mUserEntry_apfnHandlers_Idle;

	// Report this value to host
	mUserEntry_ptHostInfo->pfnCallback_OnMore( wValue );
}

void UserEntry_Handler_OnDigit_1B( BYTE byIn )
// User has typed within range 0 through 9
{
	// Store this value
	mUserEntry_byNybble1 = byIn - '0';

	// Transition to next state
	mUserEntry_pcapfnCurrentState = mUserEntry_apfnHandlers_0B;
}

void UserEntry_Handler_OnDigit_0B( BYTE byIn )
// User has typed within range 0 through 9
{	WORD wValue;

	// Store this value
	mUserEntry_byNybble0 = byIn - '0';

	// Format the completed value
	wValue = UserEntry_FormatBYTE_10();
	
	// Transition to next state
	mUserEntry_pcapfnCurrentState = mUserEntry_apfnHandlers_Idle;

	// Report this value to host
	mUserEntry_ptHostInfo->pfnCallback_OnMore( wValue );
}

void UserEntry_Handler_OnUpperAF_3W( BYTE byIn )
// User has typed within range A through F
{
	// Store this value
	mUserEntry_byNybble3 = byIn + 10 - 'A';

	// Transition to next state
	mUserEntry_pcapfnCurrentState = mUserEntry_apfnHandlers_2W;
}

void UserEntry_Handler_OnUpperAF_2W( BYTE byIn )
// User has typed within range A through F
{
	// Store this value
	mUserEntry_byNybble2 = byIn + 10 - 'A';

	// Transition to next state
	mUserEntry_pcapfnCurrentState = mUserEntry_apfnHandlers_1W;
}

void UserEntry_Handler_OnUpperAF_1W( BYTE byIn )
// User has typed within range A through F
{
	// Store this value
	mUserEntry_byNybble1 = byIn + 10 - 'A';

	// Transition to next state
	mUserEntry_pcapfnCurrentState = mUserEntry_apfnHandlers_0W;
}

void UserEntry_Handler_OnUpperAF_0W( BYTE byIn )
// User has typed within range A through F
{	WORD wValue;

	// Store this value
	mUserEntry_byNybble0 = byIn + 10 - 'A';

	// Format the completed value
	wValue = UserEntry_FormatWORD_3210();

	// Transition to next state
	mUserEntry_pcapfnCurrentState = mUserEntry_apfnHandlers_Idle;
	
	// Report this value to host
	mUserEntry_ptHostInfo->pfnCallback_OnMore( wValue );
}

void UserEntry_Handler_OnUpperAF_1B( BYTE byIn )
// User has typed within range A through F
{
	// Store this value
	mUserEntry_byNybble1 = byIn + 10 - 'A';

	// Transition to next state
	mUserEntry_pcapfnCurrentState = mUserEntry_apfnHandlers_0B;
}

void UserEntry_Handler_OnUpperAF_0B( BYTE byIn )
// User has typed within range A through F
{	WORD wValue;

	// Store this value
	mUserEntry_byNybble0 = byIn + 10 - 'A';

	// Format the completed value
	wValue = UserEntry_FormatBYTE_10();

	// Transition to next state
	mUserEntry_pcapfnCurrentState = mUserEntry_apfnHandlers_Idle;
	
	// Report this value to host
	mUserEntry_ptHostInfo->pfnCallback_OnMore( wValue );
}

void UserEntry_Handler_OnLowerAF_3W( BYTE byIn )
// User has typed within range a through f
{
	// Store this value
	mUserEntry_byNybble3 = byIn + 10 - 'a';

	// Transition to next state
	mUserEntry_pcapfnCurrentState = mUserEntry_apfnHandlers_2W;
}

void UserEntry_Handler_OnLowerAF_2W( BYTE byIn )
// User has typed within range a through f
{
	// Store this value
	mUserEntry_byNybble2 = byIn + 10 - 'a';

	// Transition to next state
	mUserEntry_pcapfnCurrentState = mUserEntry_apfnHandlers_1W;
}

void UserEntry_Handler_OnLowerAF_1W( BYTE byIn )
// User has typed within range a through f
{
	// Store this value
	mUserEntry_byNybble1 = byIn + 10 - 'a';

	// Transition to next state
	mUserEntry_pcapfnCurrentState = mUserEntry_apfnHandlers_0W;
}

void UserEntry_Handler_OnLowerAF_0W( BYTE byIn )
// User has typed within range a through f
{	WORD wValue;

	// Store this value
	mUserEntry_byNybble0 = byIn + 10 - 'a';

	// Format the completed value
	wValue = UserEntry_FormatWORD_3210();

	// Transition to next state
	mUserEntry_pcapfnCurrentState = mUserEntry_apfnHandlers_Idle;
	
	// Report this value to host
	mUserEntry_ptHostInfo->pfnCallback_OnMore( wValue );
}

void UserEntry_Handler_OnLowerAF_1B( BYTE byIn )
// User has typed within range a through f
{
	// Store this value
	mUserEntry_byNybble1 = byIn + 10 - 'a';

	// Transition to next state
	mUserEntry_pcapfnCurrentState = mUserEntry_apfnHandlers_0B;
}

void UserEntry_Handler_OnLowerAF_0B( BYTE byIn )
// User has typed within range a through f
{	WORD wValue;

	// Store this value
	mUserEntry_byNybble0 = byIn + 10 - 'a';

	// Format the completed value
	wValue = UserEntry_FormatBYTE_10();
	
	// Transition to next state
	mUserEntry_pcapfnCurrentState = mUserEntry_apfnHandlers_Idle;

	// Report this value to host
	mUserEntry_ptHostInfo->pfnCallback_OnMore( wValue );
}

void UserEntry_Handler_OnMore_2W( BYTE byIn )
// User has commanded More with data as-is
{	WORD wValue;

	// Calculate the current value
	wValue = UserEntry_FormatWORD_3();
	
	// Transition to next state
	mUserEntry_pcapfnCurrentState = mUserEntry_apfnHandlers_Idle;

	// Call the more callback
	mUserEntry_ptHostInfo->pfnCallback_OnMore( wValue );
}

void UserEntry_Handler_OnMore_1W( BYTE byIn )
// User has commanded More with data as-is
{	WORD wValue;

	// Calculate the current value
	wValue = UserEntry_FormatWORD_32();
	
	// Transition to next state
	mUserEntry_pcapfnCurrentState = mUserEntry_apfnHandlers_Idle;

	// Call the more callback
	mUserEntry_ptHostInfo->pfnCallback_OnMore( wValue );
}

void UserEntry_Handler_OnMore_0W( BYTE byIn )
// User has commanded More with data as-is
{	WORD wValue;

	// Calculate the current value
	wValue = UserEntry_FormatWORD_321();
	
	// Transition to next state
	mUserEntry_pcapfnCurrentState = mUserEntry_apfnHandlers_Idle;

	// Call the more callback
	mUserEntry_ptHostInfo->pfnCallback_OnMore( wValue );
}

void UserEntry_Handler_OnMore_0B( BYTE byIn )
// User has commanded More with data as-is
{	WORD wValue;

	// Calculate the current value
	wValue = UserEntry_FormatBYTE_1();
	
	// Transition to next state
	mUserEntry_pcapfnCurrentState = mUserEntry_apfnHandlers_Idle;

	// Call the more callback
	mUserEntry_ptHostInfo->pfnCallback_OnMore( wValue );
}

void UserEntry_Handler_OnBackspace_Ignore( BYTE byIn )
// User typed backspace
{	
	// Ignore for this state
	// Queue a warning beep
	UCA0_UART_QueueTxChar( CHAR_BELL );
}

void UserEntry_Handler_OnBackspace_2W( BYTE byIn )
// User typed backspace
{	
	// Erase the terminal character
	UCA0_UART_QueueTxErasingBackspace();

	// Revert to previous state
	mUserEntry_pcapfnCurrentState = mUserEntry_apfnHandlers_3W;
}

void UserEntry_Handler_OnBackspace_1W( BYTE byIn )
// User typed backspace
{	
	// Erase the terminal character
	UCA0_UART_QueueTxErasingBackspace();

	// Revert to previous state
	mUserEntry_pcapfnCurrentState = mUserEntry_apfnHandlers_2W;
}

void UserEntry_Handler_OnBackspace_0W( BYTE byIn )
// User typed backspace
{	
	// Erase the terminal character
	UCA0_UART_QueueTxErasingBackspace();

	// Revert to previous state
	mUserEntry_pcapfnCurrentState = mUserEntry_apfnHandlers_1W;
}

void UserEntry_Handler_OnBackspace_0B( BYTE byIn )
// User typed backspace
{	
	// Erase the terminal character
	UCA0_UART_QueueTxErasingBackspace();

	// Revert to previous state
	mUserEntry_pcapfnCurrentState = mUserEntry_apfnHandlers_1B;
}

void UserEntry_Handler_OnDone_3W( BYTE byIn )
// User has commanded Done with data as-is
{	
	// Transition to next state
	mUserEntry_pcapfnCurrentState = mUserEntry_apfnHandlers_Idle;

	// Call the done callback
	mUserEntry_ptHostInfo->pfnCallback_OnDone( DEFAULT_WORD );
}
	
void UserEntry_Handler_OnDone_2W( BYTE byIn )
// User has commanded Done with data as-is
{	WORD wValue;

	// Calculate the current value
	wValue = UserEntry_FormatWORD_3();
	
	// Transition to next state
	mUserEntry_pcapfnCurrentState = mUserEntry_apfnHandlers_Idle;

	// Call the done callback
	mUserEntry_ptHostInfo->pfnCallback_OnDone( wValue );
}

void UserEntry_Handler_OnDone_1W( BYTE byIn )
// User has commanded Done with data as-is
{	WORD wValue;

	// Calculate the current value
	wValue = UserEntry_FormatWORD_32();
	
	// Transition to next state
	mUserEntry_pcapfnCurrentState = mUserEntry_apfnHandlers_Idle;

	// Call the done callback
	mUserEntry_ptHostInfo->pfnCallback_OnDone( wValue );
}

void UserEntry_Handler_OnDone_0W( BYTE byIn )
// User has commanded Done with data as-is
{	WORD wValue;

	// Calculate the current value
	wValue = UserEntry_FormatWORD_321();
	
	// Transition to next state
	mUserEntry_pcapfnCurrentState = mUserEntry_apfnHandlers_Idle;

	// Call the done callback
	mUserEntry_ptHostInfo->pfnCallback_OnDone( wValue );
}

void UserEntry_Handler_OnDone_1B( BYTE byIn )
// User has commanded Done with data as-is
{
	// Transition to next state
	mUserEntry_pcapfnCurrentState = mUserEntry_apfnHandlers_Idle;

	// Call the done callback
	mUserEntry_ptHostInfo->pfnCallback_OnDone( DEFAULT_BYTE );
}

void UserEntry_Handler_OnDone_0B( BYTE byIn )
// User has commanded Done with data as-is
{	WORD wValue;

	// Calculate the current value
	wValue = UserEntry_FormatBYTE_1();
	
	// Transition to next state
	mUserEntry_pcapfnCurrentState = mUserEntry_apfnHandlers_Idle;

	// Call the done callback
	mUserEntry_ptHostInfo->pfnCallback_OnDone( wValue );
}

void UserEntry_Handler_OnEscape( BYTE byIn )
// User has cancelled entry
{
	// Transition to next state
	mUserEntry_pcapfnCurrentState = mUserEntry_apfnHandlers_Idle;

	// Call the abandon callback
	mUserEntry_ptHostInfo->pfnCallback_OnAbandon();
}

